unit ICWhatSubs;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module contains routines that are common to both of the List Manager based
	Whats, namely File Map What and the Helpers What.  These two whats have so much
	block copied code that it proved relatively easy to move out chunks of them into
	this module.
	
	The API to this module isn't particularly well designed.  It's basically a bunch
	of common routines collected together.
*)

interface

	uses
		Types,
		QuickDraw,

		ICWindowGlobals;

	const
		kListCellHeight = 36;
				
	procedure DrawListIcon(iconType : OSType; iconCreator : OSType;
											drawSelected : Boolean; theCellRect : Rect;
											var currentXOffset : integer; iconRegion : RgnHandle);
		(* Draws the file icon associated with a list cell given it's type
			and creator. currentXOffset is adjusted to reflect the size
			of the drawn icon. iconRegion is updated to contain the
			region of the icon drawn.

			This routine assumes that the System 7 icon utilities are
			present and that iconRegion is not nil.
		*)

	procedure HighlightListCell(theCellRect : Rect; iconRegion : RgnHandle);
		(* Draws the highlighting for a list cell by first calculating the
			highlight region based on the cell's rectangle, and then subtracting
			away the iconRegion.
		*)

	procedure ListUserItemUpdateProc (dlg: DialogPtr; item: integer);
		(* This routine is called by the Dialog Manager to update
			the main list user item in the File Mappings dialog. It is installed
			as the user item update proc by the What's Open routine.
			It basically calls straight through to the List Manager to do the
			job.
		*)

	function WhatCursorListItem (wt: WindowType; item: integer; pt: Point; cursorid: integer): OSStatus;
		(* The list Whats (File Map and Helpers) overrides the default cursor handling because
			we want to make sure that the cursor is the big plus within the body of the list,
			but the normal arrow within the scroll bar. Both of these overrides call straight
			through to here because they perform an identical function.
		*)

implementation

	uses
		Icons,
		
		ICMiscSubs,
		ICDialogs,
		ICIconSuites;

	procedure DrawListIcon(iconType : OSType; iconCreator : OSType;
											drawSelected : Boolean; theCellRect : Rect;
											var currentXOffset : integer; iconRegion : RgnHandle);
		(* See comment in interface part. *)
		var
			junk: OSStatus;
			iconRect: Rect;
			iconSuiteH: Handle;
			transfer: integer;
	begin
		(* Calculate the destination rectangle for the icon plot. *)
		iconRect := theCellRect;
		iconRect.bottom := iconRect.top + 32;
		iconRect.right := iconRect.left + 32;
		OffsetRect(iconRect, (kListCellHeight - 32) div 2, (kListCellHeight - 32) div 2);

		(* Get the icon data, and plot it. *)
		junk := GetDTDBIconSuiteCached(iconType, iconCreator, iconSuiteH);
		if iconSuiteH <> nil then begin
			if drawSelected then begin
				transfer := ttSelected;
			end else begin
				transfer := ttNone;
			end; (* if *)
			junk := PlotIconSuite(iconRect, atNone, transfer, iconSuiteH);
			junk := IconSuiteToRgn(iconRegion, iconRect, atNone, iconSuiteH);
		end; (* if *)
		
		currentXOffset := currentXOffset + kListCellHeight;
	end; (* DrawListIcon *)
	
	procedure HighlightListCell(theCellRect : Rect; iconRegion : RgnHandle);
		(* See comment in interface part. *)
		var
			highLightRegion: RgnHandle;
	begin
		highLightRegion := NewRgn;
		if highLightRegion <> nil then begin
			RectRgn(highLightRegion, theCellRect);
			MagicMarkerMode;
			if iconRegion <> nil then begin
				XorRgn(iconRegion, highLightRegion, highLightRegion);
			end; (* if *)
			InvertRgn(highLightRegion);
			DisposeRgn(highLightRegion);
		end; (* if *)
	end; (* HighlightListCell *)

	procedure ListUserItemUpdateProc (dlg: DialogPtr; item: integer);
		(* See comment in interface part. *)
		var
			listItemRect: Rect;
	begin
		GetDItemRect(dlg, item, listItemRect);
		PenNormal;
		InsetRect(listItemRect, -1, -1);
		FrameRect(listItemRect);
		LUpdate(dlg^.visRgn, ListHandle(gWindows[MapWindowPtrToWindowType(dlg)].items[item]^.whatSpareData));
	end; (* ListUserItemUpdateProc *)
		
	function WhatCursorListItem (wt: WindowType; item: integer; pt: Point; cursorid: integer): OSStatus;
		(* See comment in interface part. *)
		var
			listItemRect: Rect;
	begin
		GetDItemRect(gWindows[wt].window, item, listItemRect);
		listItemRect.right := listItemRect.right - 15;
		if PtInRect(pt, listItemRect) then begin
			SetCursor(GetCursor(cursorid)^^);
		end else begin
			InitCursor;
		end; (* if *)
		WhatCursorListItem := noErr;
	end; (* WhatCursorListItem *)

end. (* ICWhatSubs *)